@@ -26,7 +26,9 @@
 /* For section and place type enums */
 #include "nautilusgtkplacessidebarprivate.h"
 
+#ifdef HAVE_CLOUDPROVIDERS
 #include <cloudproviders.h>
+#endif
 
 struct _NautilusGtkSidebarRow
 {
@@ -83,47 +85,7 @@ enum
 
 static GParamSpec *properties [LAST_PROP];
 
-static void
-dummy_callback (NautilusFile *file,
-                gpointer      callback_data)
-{
-  /* No-op. Just a tag for nautilus_file_cancel_call_when_ready() to match. */
-}
-
-static void
-on_file_changed (NautilusGtkSidebarRow *self)
-{
-  g_return_if_fail (NAUTILUS_IS_GTK_SIDEBAR_ROW (self));
-  g_return_if_fail (NAUTILUS_IS_FILE (self->file));
-  if (nautilus_file_is_gone (self->file))
-    {
-      g_signal_handlers_disconnect_by_func (self->file, on_file_changed, self);
-      nautilus_file_cancel_call_when_ready (self->file, dummy_callback, self);
-      g_clear_object (&self->file);
-    }
-}
-
-static void
-ensure_connected_file (NautilusGtkSidebarRow *self)
-{
-  if (self->file != NULL)
-    return;
-
-  if (self->uri != NULL)
-    {
-      g_autoptr (NautilusFile) file = nautilus_file_get_by_uri (self->uri);
-
-      /* Self-owned file may be marked as gone, if we are rebuilding the sidebar
-       * in response to that file being unmounted. Don't keep it alive. */
-      if (file == NULL || nautilus_file_is_gone (file))
-        return;
-
-      self->file = g_steal_pointer (&file);
-      nautilus_file_call_when_ready (self->file, NAUTILUS_FILE_ATTRIBUTE_MOUNT, dummy_callback, self);
-      g_signal_connect_object (self->file, "changed",
-                               G_CALLBACK (on_file_changed), self, G_CONNECT_SWAPPED);
-    }
-}
+#ifdef HAVE_CLOUDPROVIDERS
 
 static void
 cloud_row_update (NautilusGtkSidebarRow *self)
@@ -166,6 +128,8 @@ cloud_row_update (NautilusGtkSidebarRow *self)
     g_object_unref (end_icon);
 }
 
+#endif
+
 static void
 nautilus_gtk_sidebar_row_get_property (GObject    *object,
                               guint       prop_id,
@@ -221,7 +185,6 @@ nautilus_gtk_sidebar_row_get_property (GObject    *object,
       break;
 
     case PROP_NAUTILUS_FILE:
-      ensure_connected_file (self);
       g_value_set_object (value, self->file);
       break;
 
@@ -344,7 +307,12 @@ nautilus_gtk_sidebar_row_set_property (GObject      *object,
     case PROP_URI:
       g_free (self->uri);
       self->uri = g_strdup (g_value_get_string (value));
-      ensure_connected_file (self);
+      if (self->uri != NULL)
+        {
+          self->file = nautilus_file_get_by_uri (self->uri);
+          if (self->file != NULL)
+            nautilus_file_call_when_ready (self->file, NAUTILUS_FILE_ATTRIBUTE_MOUNT, NULL, NULL);
+        }
       break;
 
     case PROP_DRIVE:
@@ -360,6 +328,7 @@ nautilus_gtk_sidebar_row_set_property (GObject      *object,
       break;
 
     case PROP_CLOUD_PROVIDER_ACCOUNT:
+#ifdef HAVE_CLOUDPROVIDERS
       if (self->cloud_provider_account != NULL)
         g_signal_handlers_disconnect_by_data (self->cloud_provider_account, self);
 
@@ -374,6 +343,7 @@ nautilus_gtk_sidebar_row_set_property (GObject      *object,
           g_signal_connect_swapped (self->cloud_provider_account, "notify::status-details",
                                     G_CALLBACK (cloud_row_update), self);
         }
+#endif
       break;
 
     case PROP_PLACEHOLDER:
@@ -502,9 +472,11 @@ nautilus_gtk_sidebar_row_finalize (GObject *object)
   g_clear_object (&self->drive);
   g_clear_object (&self->volume);
   g_clear_object (&self->mount);
+#ifdef HAVE_CLOUDPROVIDERS
   if (self->cloud_provider_account != NULL)
     g_signal_handlers_disconnect_by_data (self->cloud_provider_account, self);
   g_clear_object (&self->cloud_provider_account);
+#endif
 
   G_OBJECT_CLASS (nautilus_gtk_sidebar_row_parent_class)->finalize (object);
 }
