@@ -8,9 +8,14 @@
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
+#include <glib/gi18n.h>
+
 #include "nautilus-directory.h"
 #include "nautilus-dnd.h"
 #include "nautilus-file-utilities.h"
+#include "nautilus-files-view.h"
+#include "nautilus-files-view-dnd.h"
+#include "nautilus-scheme.h"
 #include "nautilus-tag-manager.h"
 
 static gboolean
@@ -72,7 +77,7 @@ append_drop_action_menu_item (GtkWidget          *menu,
     gtk_widget_set_sensitive (menu_item, sensitive);
     gtk_box_append (GTK_BOX (menu), menu_item);
 
-    gtk_style_context_add_class (gtk_widget_get_style_context (menu_item), "flat");
+    gtk_widget_add_css_class (menu_item, "flat");
 
     g_object_set_data (G_OBJECT (menu_item),
                        "action",
@@ -219,7 +224,7 @@ nautilus_dnd_get_preferred_action (NautilusFile *target_file,
         return GDK_ACTION_COPY;
     }
 
-    if (g_file_has_uri_scheme (dropped, "trash"))
+    if (g_file_has_uri_scheme (dropped, SCHEME_TRASH))
     {
         return GDK_ACTION_MOVE;
     }
@@ -235,6 +240,61 @@ nautilus_dnd_get_preferred_action (NautilusFile *target_file,
     return GDK_ACTION_COPY;
 }
 
+gboolean
+nautilus_dnd_perform_drop (NautilusFilesView *view,
+                           const GValue      *value,
+                           GdkDragAction      action,
+                           GFile             *target_location)
+{
+    g_autofree gchar *target_uri = g_file_get_uri (target_location);
+
+    if (!gdk_drag_action_is_unique (action))
+    {
+        /* TODO: Implement */
+        return FALSE;
+    }
+    else if (G_VALUE_HOLDS (value, G_TYPE_STRING))
+    {
+        nautilus_files_view_handle_text_drop (view,
+                                              g_value_get_string (value),
+                                              target_uri, action);
+        return TRUE;
+    }
+    else if (G_VALUE_HOLDS (value, GDK_TYPE_TEXTURE))
+    {
+        g_autofree char *dest_uri = g_file_get_uri (target_location);
+        GdkTexture *texture = g_value_get_object (value);
+
+        nautilus_file_view_save_image_from_texture (view, texture, dest_uri, _("Dropped image"));
+    }
+    else if (G_VALUE_HOLDS (value, GDK_TYPE_FILE_LIST))
+    {
+        GSList *source_file_list = g_value_get_boxed (value);
+        GList *source_uri_list = NULL;
+
+        if (source_file_list == NULL)
+        {
+            return FALSE;
+        }
+
+        for (GSList *l = source_file_list; l != NULL; l = l->next)
+        {
+            source_uri_list = g_list_prepend (source_uri_list, g_file_get_uri (l->data));
+        }
+        source_uri_list = g_list_reverse (source_uri_list);
+
+        nautilus_files_view_drop_proxy_received_uris (view,
+                                                      source_uri_list,
+                                                      target_uri,
+                                                      action);
+        g_list_free_full (source_uri_list, g_free);
+
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
 #define MAX_DRAWN_DRAG_ICONS 10
 
 GdkPaintable *
