@@ -32,6 +32,7 @@
 #include "nautilus-file-undo-manager.h"
 #include "nautilus-batch-rename-dialog.h"
 #include "nautilus-batch-rename-utilities.h"
+#include "nautilus-scheme.h"
 #include "nautilus-tag-manager.h"
 
 
@@ -474,8 +475,8 @@ ext_strings_func (NautilusFileUndoInfo  *info,
     }
     else if (op_type == NAUTILUS_FILE_UNDO_OP_RESTORE_FROM_TRASH)
     {
-        *undo_label = g_strdup (_("_Undo Restore from Trash"));
-        *redo_label = g_strdup (_("_Redo Restore from Trash"));
+        *undo_label = g_strdup (_("_Undo Restore From Trash"));
+        *redo_label = g_strdup (_("_Redo Restore From Trash"));
 
         if (count > 1)
         {
@@ -843,8 +844,8 @@ create_strings_func (NautilusFileUndoInfo  *info,
     {
         *redo_description = g_strdup_printf (_("Create new file “%s” from template "), name);
 
-        *undo_label = g_strdup (_("_Undo Create from Template"));
-        *redo_label = g_strdup (_("_Redo Create from Template"));
+        *undo_label = g_strdup (_("_Undo Create From Template"));
+        *redo_label = g_strdup (_("_Redo Create From Template"));
     }
     else
     {
@@ -1712,7 +1713,7 @@ trash_retrieve_files_to_restore_thread (GTask        *task,
     to_restore = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal,
                                         g_object_unref, g_object_unref);
 
-    trash = g_file_new_for_uri ("trash:///");
+    trash = g_file_new_for_uri (SCHEME_TRASH ":///");
 
     enumerator = g_file_enumerate_children (trash,
                                             G_FILE_ATTRIBUTE_STANDARD_NAME ","
