@@ -52,8 +52,7 @@ typedef struct NautilusFile NautilusFile;
 typedef enum {
 	/* These may be set as default-sort-order. When touching this, make sure to
 	 * keep the values in sync with the "org.gnome.nautilus.SortOrder" enum in the
-	 * `data/org.gnome.nautilus.gschema.xml` schemas file, and the attributes[]
-	 * array in `src/nautilus-list-view.c`.
+	 * `data/org.gnome.nautilus.gschema.xml` schemas file.
 	 */
 	NAUTILUS_FILE_SORT_BY_DISPLAY_NAME = 0,
 	NAUTILUS_FILE_SORT_BY_SIZE = 1,
@@ -487,11 +486,11 @@ gboolean                nautilus_file_list_are_all_folders              (const G
 /* Debugging */
 void                    nautilus_file_dump                              (NautilusFile                   *file);
 
-typedef struct NautilusFileDetails NautilusFileDetails;
+typedef struct NautilusFilePrivate NautilusFilePrivate;
 
 struct NautilusFile {
 	GObject parent_slot;
-	NautilusFileDetails *details;
+	NautilusFilePrivate *details;
 };
 
 /* This is actually a "protected" type, but it must be here so we can
