@@ -62,6 +62,18 @@ struct _NautilusListBasePrivate
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (NautilusListBase, nautilus_list_base, NAUTILUS_TYPE_FILES_VIEW)
 
+enum
+{
+    PROP_0,
+    PROP_ICON_SIZE,
+    N_PROPS
+};
+
+static GParamSpec *properties[N_PROPS] = { NULL, };
+
+static void real_add_files (NautilusFilesView *files_view,
+                            GList             *files);
+
 static const char *
 get_sort_attribute_from_sort_type (NautilusFileSortType sort_type)
 {
@@ -128,7 +140,7 @@ get_view_item (GListModel *model,
     g_autoptr (GtkTreeListRow) row = g_list_model_get_item (model, position);
 
     g_return_val_if_fail (GTK_IS_TREE_LIST_ROW (row), NULL);
-    return NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (GTK_TREE_LIST_ROW (row)));
+    return NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (row));
 }
 
 static char *
@@ -196,25 +208,6 @@ update_sort_order_from_metadata_and_preferences (NautilusListBase *self)
                                                        reversed));
 }
 
-void
-nautilus_list_base_set_icon_size (NautilusListBase *self,
-                                  gint              icon_size)
-{
-    GListModel *model;
-    guint n_items;
-
-    model = G_LIST_MODEL (nautilus_list_base_get_model (self));
-
-    n_items = g_list_model_get_n_items (model);
-    for (guint i = 0; i < n_items; i++)
-    {
-        g_autoptr (NautilusViewItem) current_item = NULL;
-
-        current_item = get_view_item (model, i);
-        nautilus_view_item_set_icon_size (current_item, icon_size);
-    }
-}
-
 void
 set_focus_item (NautilusListBase *self,
                 NautilusViewItem *item)
@@ -684,7 +677,7 @@ get_preferred_action (NautilusFile *target_file,
             action = nautilus_dnd_get_preferred_action (target_file, NULL);
         }
     }
-    else if (G_VALUE_HOLDS (value, G_TYPE_STRING))
+    else if (G_VALUE_HOLDS (value, G_TYPE_STRING) || G_VALUE_HOLDS (value, GDK_TYPE_TEXTURE))
     {
         action = GDK_ACTION_COPY;
     }
@@ -914,7 +907,7 @@ on_view_drop (GtkDropTarget *target,
 }
 
 void
-setup_cell_common (GtkListItem      *listitem,
+setup_cell_common (GObject          *listitem,
                    NautilusViewCell *cell)
 {
     GtkExpression *expression;
@@ -925,8 +918,6 @@ setup_cell_common (GtkListItem      *listitem,
     expression = gtk_property_expression_new (GTK_TYPE_TREE_LIST_ROW, expression, "item");
     gtk_expression_bind (expression, cell, "item", listitem);
 
-    gtk_list_item_set_child (listitem, GTK_WIDGET (cell));
-
     controller = GTK_EVENT_CONTROLLER (gtk_gesture_click_new ());
     gtk_widget_add_controller (GTK_WIDGET (cell), controller);
     gtk_event_controller_set_propagation_phase (controller, GTK_PHASE_BUBBLE);
@@ -950,7 +941,7 @@ setup_cell_common (GtkListItem      *listitem,
     drop_target = gtk_drop_target_new (G_TYPE_INVALID, GDK_ACTION_ALL);
     gtk_drop_target_set_preload (drop_target, TRUE);
     /* TODO: Implement GDK_TYPE_STRING */
-    gtk_drop_target_set_gtypes (drop_target, (GType[2]) { GDK_TYPE_FILE_LIST, G_TYPE_STRING }, 2);
+    gtk_drop_target_set_gtypes (drop_target, (GType[3]) { GDK_TYPE_TEXTURE, GDK_TYPE_FILE_LIST, G_TYPE_STRING }, 3);
     g_signal_connect (drop_target, "enter", G_CALLBACK (on_item_drag_enter), cell);
     g_signal_connect (drop_target, "notify::value", G_CALLBACK (on_item_drag_value_notify), cell);
     g_signal_connect (drop_target, "leave", G_CALLBACK (on_item_drag_leave), cell);
@@ -1074,11 +1065,53 @@ real_file_changed (NautilusFilesView *files_view,
                    NautilusDirectory *directory)
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
+    g_autoptr (NautilusFile) directory_as_file = NULL;
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
     NautilusViewItem *item;
 
-    item = nautilus_view_model_get_item_from_file (priv->model, file);
-    nautilus_view_item_file_changed (item);
+    directory_as_file = nautilus_directory_get_corresponding_file (directory);
+    if (file == directory_as_file)
+    {
+        /* We don't care about changes to the current directory itself here, so
+         * silently ignore it. This happens only with self-owned files.*/
+        return;
+    }
+
+    item = nautilus_view_model_find_item_for_file (priv->model, file);
+    if (item != NULL)
+    {
+        nautilus_view_item_file_changed (item);
+    }
+    else
+    {
+        /* When a file that was hidden is not hidden anymore (e.g. undoing the
+         * rename operation which made it hidden), we get a change notification
+         * for a file that's not in our model. Let's add it then. */
+        real_add_files (files_view, &(GList){file, NULL});
+    }
+}
+static gboolean
+is_ancestor_selected (GtkTreeListRow *row,
+                      GtkBitset      *selection)
+{
+    g_autoptr (GtkTreeListRow) parent = gtk_tree_list_row_get_parent (row);
+    GtkTreeListRow *grandparent;
+
+    /* Walk up the tree looking for a selected ancestor. */
+    while (parent != NULL)
+    {
+        guint parent_position = gtk_tree_list_row_get_position (parent);
+        if (gtk_bitset_contains (selection, parent_position))
+        {
+            return TRUE;
+        }
+
+        grandparent = gtk_tree_list_row_get_parent (parent);
+        g_object_unref (parent);
+        parent = grandparent;
+    }
+
+    return FALSE;
 }
 
 static GList *
@@ -1087,39 +1120,36 @@ get_selection (NautilusFilesView *files_view,
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
-    g_autoptr (GtkSelectionFilterModel) selection = NULL;
-    guint n_selected;
+    g_autoptr (GtkBitset) selection = NULL;
+    GtkBitsetIter iter;
+    guint i;
     GList *selected_files = NULL;
 
-    selection = gtk_selection_filter_model_new (GTK_SELECTION_MODEL (priv->model));
-    n_selected = g_list_model_get_n_items (G_LIST_MODEL (selection));
-    for (guint i = 0; i < n_selected; i++)
+    selection = gtk_selection_model_get_selection (GTK_SELECTION_MODEL (priv->model));
+
+    for (gtk_bitset_iter_init_last (&iter, selection, &i);
+         gtk_bitset_iter_is_valid (&iter);
+         gtk_bitset_iter_previous (&iter, &i))
     {
+        g_autoptr (GtkTreeListRow) row = NULL;
         g_autoptr (NautilusViewItem) item = NULL;
         NautilusFile *file;
 
-        item = get_view_item (G_LIST_MODEL (selection), i);
+        row = GTK_TREE_LIST_ROW (g_list_model_get_item (G_LIST_MODEL (priv->model), i));
+
+        if (for_file_transfer && is_ancestor_selected (row, selection))
+        {
+            /* If an ancestor is already selected, don't include its descendants
+             * in the selection of files to copy/move. */
+            continue;
+        }
+
+        item = NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (row));
         file = nautilus_view_item_get_file (item);
 
-        if (for_file_transfer)
-        {
-            /* If the parent is already selected don't include the child. */
-            g_autoptr (NautilusFile) parent = nautilus_file_get_parent (file);
-            NautilusViewItem *parent_item;
-            guint parent_pos;
-
-            parent_item = nautilus_view_model_get_item_from_file (priv->model, parent);
-            parent_pos = nautilus_view_model_get_index (priv->model, parent_item);
-            if (gtk_selection_model_is_selected (GTK_SELECTION_MODEL (priv->model), parent_pos))
-            {
-                continue;
-            }
-        }
         selected_files = g_list_prepend (selected_files, g_object_ref (file));
     }
 
-    selected_files = g_list_reverse (selected_files);
-
     return selected_files;
 }
 
@@ -1154,55 +1184,30 @@ real_end_file_changes (NautilusFilesView *files_view)
 }
 
 static void
-real_remove_file (NautilusFilesView *files_view,
-                  NautilusFile      *file,
-                  NautilusDirectory *directory)
+real_remove_files (NautilusFilesView *files_view,
+                   GList             *files,
+                   NautilusDirectory *directory)
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
-    NautilusViewItem *item;
+    g_autoptr (GList) items = NULL;
 
-    item = nautilus_view_model_get_item_from_file (priv->model, file);
-    if (item != NULL)
-    {
-        nautilus_view_model_remove_item (priv->model, item, directory);
-        nautilus_files_view_notify_selection_changed (files_view);
-    }
-}
-
-static GQueue *
-convert_glist_to_queue (GList *list)
-{
-    GList *l;
-    GQueue *queue;
-
-    queue = g_queue_new ();
-    for (l = list; l != NULL; l = l->next)
-    {
-        g_queue_push_tail (queue, l->data);
-    }
-
-    return queue;
-}
-
-static GQueue *
-convert_files_to_items (NautilusListBase *self,
-                        GQueue           *files)
-{
-    GList *l;
-    GQueue *models;
-
-    models = g_queue_new ();
-    for (l = g_queue_peek_head_link (files); l != NULL; l = l->next)
+    for (GList *l = files; l != NULL; l = l->next)
     {
         NautilusViewItem *item;
 
-        item = nautilus_view_item_new (NAUTILUS_FILE (l->data),
-                                       nautilus_list_base_get_icon_size (self));
-        g_queue_push_tail (models, item);
+        item = nautilus_view_model_find_item_for_file (priv->model, l->data);
+
+        if (item != NULL)
+        {
+            items = g_list_prepend (items, item);
+        }
     }
 
-    return models;
+    if (items != NULL)
+    {
+        nautilus_view_model_remove_items (priv->model, items, directory);
+    }
 }
 
 static void
@@ -1211,11 +1216,11 @@ real_set_selection (NautilusFilesView *files_view,
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
-    g_autoptr (GQueue) selection_files = NULL;
-    g_autoptr (GQueue) selection_items = NULL;
+    g_autoptr (GList) files_to_find = g_list_copy (selection);
     g_autoptr (GtkBitset) update_set = NULL;
     g_autoptr (GtkBitset) new_selection_set = NULL;
     g_autoptr (GtkBitset) old_selection_set = NULL;
+    guint n_items;
 
     old_selection_set = gtk_selection_model_get_selection (GTK_SELECTION_MODEL (priv->model));
     /* We aren't allowed to modify the actual selection bitset */
@@ -1223,19 +1228,37 @@ real_set_selection (NautilusFilesView *files_view,
     new_selection_set = gtk_bitset_new_empty ();
 
     /* Convert file list into set of model indices */
-    selection_files = convert_glist_to_queue (selection);
-    selection_items = nautilus_view_model_get_items_from_files (priv->model, selection_files);
-    for (GList *l = g_queue_peek_head_link (selection_items); l != NULL; l = l->next)
+    n_items = g_list_model_get_n_items (G_LIST_MODEL (priv->model));
+    for (guint position = 0; position < n_items; position++)
     {
-        gtk_bitset_add (new_selection_set,
-                        nautilus_view_model_get_index (priv->model, l->data));
+        g_autoptr (NautilusViewItem) item = get_view_item (G_LIST_MODEL (priv->model), position);
+
+        GList *link = g_list_find (files_to_find, nautilus_view_item_get_file (item));
+        if (link != NULL)
+        {
+            /* Found item to select */
+            gtk_bitset_add (new_selection_set, position);
+
+            /* Remove found file from the list of files yet to find. */
+            files_to_find = g_list_delete_link (files_to_find, link);
+            if (files_to_find == NULL)
+            {
+                /* We've matched everything... */
+                break;
+            }
+        }
     }
+    /* ...have we not? */
+    g_warn_if_fail (files_to_find == NULL);
 
     /* Set focus on the first selected row. */
-    if (!g_queue_is_empty (selection_items))
+    if (!gtk_bitset_is_empty (new_selection_set))
     {
-        NautilusViewItem *item = g_queue_peek_head (selection_items);
-        set_focus_item (self, item);
+        g_autoptr (NautilusViewItem) first_selected_item = NULL;
+        guint first_position = gtk_bitset_get_nth (new_selection_set, 0);
+
+        first_selected_item = get_view_item (G_LIST_MODEL (priv->model), first_position);
+        set_focus_item (self, first_selected_item);
     }
 
     gtk_bitset_union (update_set, new_selection_set);
@@ -1290,7 +1313,7 @@ get_first_selected_item (NautilusListBase *self)
     }
 
     file = NAUTILUS_FILE (selection->data);
-    item = nautilus_view_model_get_item_from_file (priv->model, file);
+    item = nautilus_view_model_find_item_for_file (priv->model, file);
 
     return nautilus_view_model_get_index (priv->model, item);
 }
@@ -1328,7 +1351,7 @@ on_clipboard_contents_received (GObject      *source_object,
 
     for (GList *l = priv->cut_files; l != NULL; l = l->next)
     {
-        item = nautilus_view_model_get_item_from_file (priv->model, l->data);
+        item = nautilus_view_model_find_item_for_file (priv->model, l->data);
         if (item != NULL)
         {
             nautilus_view_item_set_cut (item, FALSE);
@@ -1354,7 +1377,7 @@ on_clipboard_contents_received (GObject      *source_object,
 
     for (GList *l = priv->cut_files; l != NULL; l = l->next)
     {
-        item = nautilus_view_model_get_item_from_file (priv->model, l->data);
+        item = nautilus_view_model_find_item_for_file (priv->model, l->data);
         if (item != NULL)
         {
             nautilus_view_item_set_cut (item, TRUE);
@@ -1477,7 +1500,7 @@ scroll_to_file_on_idle (ScrollToFileData *data)
     priv->scroll_to_file_handle_id = 0;
 
     file = nautilus_file_get_existing_by_uri (data->uri);
-    item = nautilus_view_model_get_item_from_file (priv->model, file);
+    item = nautilus_view_model_find_item_for_file (priv->model, file);
     g_return_val_if_fail (item != NULL, G_SOURCE_REMOVE);
 
     i = nautilus_view_model_get_index (priv->model, item);
@@ -1511,11 +1534,9 @@ real_add_files (NautilusFilesView *files_view,
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
-    g_autoptr (GQueue) files_queue = NULL;
-    g_autoqueue (NautilusViewItem) items = NULL;
+    g_autolist (NautilusViewItem) items = NULL;
 
-    files_queue = convert_glist_to_queue (files);
-    items = convert_files_to_items (self, files_queue);
+    items = g_list_copy_deep (files, (GCopyFunc) nautilus_view_item_new, NULL);
     nautilus_view_model_add_items (priv->model, items);
 }
 
@@ -1566,41 +1587,37 @@ real_compute_rename_popover_pointing_to (NautilusFilesView *files_view)
     return get_rectangle_for_item_ui (self, item_ui);
 }
 
-/**
- * Gets area to popup the context menu from, ensuring it is currently visible.
- *
- * Don't use this if context menu is triggered by a pointing device (in which
- * case it should popup from the position of the pointer). Instead, this is
- * meant for non-pointer case, such as using the Menu key from the keyboard.
- **/
 static GdkRectangle *
 real_reveal_for_selection_context_menu (NautilusFilesView *files_view)
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
-    g_autoptr (GtkBitset) selection = NULL;
+    g_autoptr (GtkSelectionFilterModel) selection = NULL;
+    guint n_selected;
+    GtkWidget *focus_child;
     guint i;
-    g_autoptr (NautilusViewItem) item = NULL;
     GtkWidget *item_ui;
 
-    selection = gtk_selection_model_get_selection (GTK_SELECTION_MODEL (priv->model));
-    g_return_val_if_fail (!gtk_bitset_is_empty (selection), NULL);
+    selection = gtk_selection_filter_model_new (GTK_SELECTION_MODEL (priv->model));
+    n_selected = g_list_model_get_n_items (G_LIST_MODEL (selection));
+    g_return_val_if_fail (n_selected > 0, NULL);
 
-    /* If multiple items are selected, we need to pick one. The ideal choice is
-     * the currently focused item, if it is part of the selection.
-     *
-     * (Sadly, that's not currently possible, because GTK doesn't let us know
-     * the focus position: https://gitlab.gnome.org/GNOME/gtk/-/issues/2891 )
-     *
+    /* Get the focused item_ui, if selected.
      * Otherwise, get the selected item_ui which is sorted the lowest.*/
-    i = gtk_bitset_get_minimum (selection);
+    focus_child = gtk_widget_get_focus_child (nautilus_list_base_get_view_ui (self));
+    for (i = 0; i < n_selected; i++)
+    {
+        g_autoptr (NautilusViewItem) item = NULL;
 
+        item = get_view_item (G_LIST_MODEL (selection), i);
+        item_ui = nautilus_view_item_get_item_ui (item);
+        if (item_ui != NULL && gtk_widget_get_parent (item_ui) == focus_child)
+        {
+            break;
+        }
+    }
     nautilus_list_base_scroll_to_item (self, i);
 
-    item = get_view_item (G_LIST_MODEL (priv->model), i);
-    item_ui = nautilus_view_item_get_item_ui (item);
-    g_return_val_if_fail (item_ui != NULL, NULL);
-
     return get_rectangle_for_item_ui (self, item_ui);
 }
 
@@ -1656,7 +1673,6 @@ nautilus_list_base_dispose (GObject *object)
     g_clear_handle_id (&priv->hover_timer_id, g_source_remove);
 
     g_cancellable_cancel (priv->clipboard_cancellable);
-    g_clear_object (&priv->clipboard_cancellable);
 
     G_OBJECT_CLASS (nautilus_list_base_parent_class)->dispose (object);
 }
@@ -1668,6 +1684,8 @@ nautilus_list_base_finalize (GObject *object)
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
 
     g_clear_list (&priv->cut_files, g_object_unref);
+    /* Clear cancellable in finalize to prevent null usage */
+    g_clear_object (&priv->clipboard_cancellable);
 
     G_OBJECT_CLASS (nautilus_list_base_parent_class)->finalize (object);
 }
@@ -1764,18 +1782,6 @@ nautilus_list_base_focus (GtkWidget        *widget,
     gboolean no_selection;
     gboolean handled;
 
-    /* If focus is already inside the view, allow to immediately tab out of it,
-     * instead of having to cycle through every item (potentially many). */
-    if (direction == GTK_DIR_TAB_FORWARD || direction == GTK_DIR_TAB_BACKWARD)
-    {
-        GtkWidget *content_widget = nautilus_files_view_get_content_widget (NAUTILUS_FILES_VIEW (self));
-        GtkWidget *focus_widget = gtk_root_get_focus (gtk_widget_get_root (widget));
-        if (focus_widget != NULL && gtk_widget_is_ancestor (focus_widget, content_widget))
-        {
-            return FALSE;
-        }
-    }
-
     selection = nautilus_view_get_selection (NAUTILUS_VIEW (self));
     no_selection = (selection == NULL);
 
@@ -1800,6 +1806,29 @@ nautilus_list_base_focus (GtkWidget        *widget,
     return handled;
 }
 
+static void
+nautilus_list_base_get_property (GObject    *object,
+                                 guint       prop_id,
+                                 GValue     *value,
+                                 GParamSpec *pspec)
+{
+    NautilusListBase *self = NAUTILUS_LIST_BASE (object);
+
+    switch (prop_id)
+    {
+        case PROP_ICON_SIZE:
+        {
+            g_value_set_uint (value, nautilus_list_base_get_icon_size (self));
+        }
+        break;
+
+        default:
+        {
+            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        }
+    }
+}
+
 static void
 nautilus_list_base_class_init (NautilusListBaseClass *klass)
 {
@@ -1809,6 +1838,7 @@ nautilus_list_base_class_init (NautilusListBaseClass *klass)
 
     object_class->dispose = nautilus_list_base_dispose;
     object_class->finalize = nautilus_list_base_finalize;
+    object_class->get_property = nautilus_list_base_get_property;
 
     widget_class->focus = nautilus_list_base_focus;
 
@@ -1820,7 +1850,7 @@ nautilus_list_base_class_init (NautilusListBaseClass *klass)
     files_view_class->get_selection = real_get_selection;
     files_view_class->get_selection_for_file_transfer = real_get_selection_for_file_transfer;
     files_view_class->is_empty = real_is_empty;
-    files_view_class->remove_file = real_remove_file;
+    files_view_class->remove_files = real_remove_files;
     files_view_class->select_all = real_select_all;
     files_view_class->set_selection = real_set_selection;
     files_view_class->invert_selection = real_invert_selection;
@@ -1833,6 +1863,14 @@ nautilus_list_base_class_init (NautilusListBaseClass *klass)
     files_view_class->compute_rename_popover_pointing_to = real_compute_rename_popover_pointing_to;
     files_view_class->reveal_for_selection_context_menu = real_reveal_for_selection_context_menu;
     files_view_class->preview_selection_event = real_preview_selection_event;
+
+    properties[PROP_ICON_SIZE] = g_param_spec_uint ("icon-size",
+                                                    "", "",
+                                                    NAUTILUS_LIST_ICON_SIZE_SMALL,
+                                                    NAUTILUS_GRID_ICON_SIZE_EXTRA_LARGE,
+                                                    NAUTILUS_GRID_ICON_SIZE_LARGE,
+                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+    g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
 static void
@@ -1842,8 +1880,6 @@ nautilus_list_base_init (NautilusListBase *self)
     GtkWidget *content_widget;
     GtkAdjustment *vadjustment;
 
-    gtk_widget_add_css_class (GTK_WIDGET (self), "view");
-
     g_signal_connect_object (nautilus_preferences,
                              "changed::" NAUTILUS_PREFERENCES_DEFAULT_SORT_ORDER,
                              G_CALLBACK (default_sort_order_changed_callback),
@@ -1911,7 +1947,7 @@ nautilus_list_base_setup_gestures (NautilusListBase *self)
     drop_target = gtk_drop_target_new (G_TYPE_INVALID, GDK_ACTION_ALL);
     gtk_drop_target_set_preload (drop_target, TRUE);
     /* TODO: Implement GDK_TYPE_STRING */
-    gtk_drop_target_set_gtypes (drop_target, (GType[2]) { GDK_TYPE_FILE_LIST, G_TYPE_STRING }, 2);
+    gtk_drop_target_set_gtypes (drop_target, (GType[3]) { GDK_TYPE_TEXTURE, GDK_TYPE_FILE_LIST, G_TYPE_STRING }, 3);
     g_signal_connect (drop_target, "enter", G_CALLBACK (on_view_drag_enter), self);
     g_signal_connect (drop_target, "notify::value", G_CALLBACK (on_view_drag_value_notify), self);
     g_signal_connect (drop_target, "motion", G_CALLBACK (on_view_drag_motion), self);
