@@ -23,7 +23,6 @@
 #pragma once
 
 #include <gio/gio.h>
-#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include "nautilus-types.h"
@@ -31,7 +30,6 @@
 NautilusFileAttributes nautilus_mime_actions_get_required_file_attributes (void);
 
 GAppInfo *             nautilus_mime_get_default_application_for_file     (NautilusFile            *file);
-GList *                nautilus_mime_get_applications_for_file            (NautilusFile            *file);
 
 GAppInfo *             nautilus_mime_get_default_application_for_files    (GList                   *files);
 
