@@ -47,14 +47,6 @@ NautilusQuery *nautilus_query_editor_get_query    (NautilusQueryEditor *editor);
  */
 void           nautilus_query_editor_set_query    (NautilusQueryEditor *editor,
                                                    NautilusQuery       *query);
-/**
- * nautilus_query_editor_get_location:
- *
- * @editor: A #NautilusQueryEditor instance.
- *
- * Returns: (nullable) (transfer full): The location of the current search.
- */
-GFile         *nautilus_query_editor_get_location (NautilusQueryEditor *editor);
 /**
  * nautilus_query_editor_set_location:
  *
@@ -63,9 +55,14 @@ GFile         *nautilus_query_editor_get_location (NautilusQueryEditor *editor);
  */
 void           nautilus_query_editor_set_location (NautilusQueryEditor *editor,
                                                    GFile               *location);
+/**
+ * nautilus_query_editor_select_all_text:
+ *
+ * @editor: A #NautilusQueryEditor instance.
+ */
+void           nautilus_query_editor_select_all_text (NautilusQueryEditor   *editor);
 
-gboolean
-nautilus_query_editor_handle_event (NautilusQueryEditor   *self,
-                                    GtkEventControllerKey *controller,
-                                    guint                  keyval,
-                                    GdkModifierType        state);
+gboolean       nautilus_query_editor_handle_event    (NautilusQueryEditor   *self,
+                                                      GtkEventControllerKey *controller,
+                                                      guint                  keyval,
+                                                      GdkModifierType        state);
