@@ -22,7 +22,6 @@
 #include <config.h>
 #include "nautilus-search-engine-simple.h"
 
-#include "nautilus-search-engine-private.h"
 #include "nautilus-search-hit.h"
 #include "nautilus-search-provider.h"
 #include "nautilus-ui-utilities.h"
@@ -34,6 +33,7 @@
 #include <gio/gio.h>
 
 #define BATCH_SIZE 500
+#define CREATE_THREAD_DELAY_MS 500
 
 enum
 {
@@ -58,7 +58,6 @@ typedef struct
     GList *hits;
 
     NautilusQuery *query;
-
     gint processing_id;
     GMutex idle_mutex;
     /* The following data can be accessed from different threads
@@ -73,6 +72,7 @@ struct _NautilusSearchEngineSimple
 {
     GObject parent_instance;
     NautilusQuery *query;
+    guint create_thread_timeout_id;
 
     SearchThreadData *active_search;
 };
@@ -90,6 +90,7 @@ finalize (GObject *object)
 {
     NautilusSearchEngineSimple *simple = NAUTILUS_SEARCH_ENGINE_SIMPLE (object);
     g_clear_object (&simple->query);
+    g_clear_handle_id (&simple->create_thread_timeout_id, g_source_remove);
 
     G_OBJECT_CLASS (nautilus_search_engine_simple_parent_class)->finalize (object);
 }
@@ -99,7 +100,6 @@ search_thread_data_new (NautilusSearchEngineSimple *engine,
                         NautilusQuery              *query)
 {
     SearchThreadData *data;
-    GFile *location;
 
     data = g_new0 (SearchThreadData, 1);
 
@@ -107,10 +107,6 @@ search_thread_data_new (NautilusSearchEngineSimple *engine,
     data->directories = g_queue_new ();
     data->visited = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
     data->query = g_object_ref (query);
-
-    location = nautilus_query_get_location (query);
-
-    g_queue_push_tail (data->directories, location);
     data->mime_types = nautilus_query_get_mime_types (query);
 
     data->cancellable = g_cancellable_new ();
@@ -286,7 +282,7 @@ visit_directory (GFile            *dir,
 {
     g_autoptr (GPtrArray) date_range = NULL;
     NautilusQuerySearchType type;
-    NautilusQueryRecursive recursive;
+    NautilusQueryRecursive recursive_flag;
     GFileEnumerator *enumerator;
     GFileInfo *info;
     GFile *child;
@@ -316,7 +312,7 @@ visit_directory (GFile            *dir,
     }
 
     type = nautilus_query_get_search_type (data->query);
-    recursive = nautilus_query_get_recursive (data->query);
+    recursive_flag = nautilus_query_get_recursive (data->query);
     date_range = nautilus_query_get_date_range (data->query);
 
     while ((info = g_file_enumerator_next_file (enumerator, data->cancellable, NULL)) != NULL)
@@ -324,6 +320,7 @@ visit_directory (GFile            *dir,
         g_autoptr (GDateTime) mtime = NULL;
         g_autoptr (GDateTime) atime = NULL;
         g_autoptr (GDateTime) ctime = NULL;
+        gboolean recursive = FALSE;
 
         display_name = g_file_info_get_display_name (info);
         if (display_name == NULL)
@@ -429,10 +426,29 @@ visit_directory (GFile            *dir,
             send_batch_in_idle (data);
         }
 
-        if (recursive != NAUTILUS_QUERY_RECURSIVE_NEVER &&
-            g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY &&
-            is_recursive_search (NAUTILUS_SEARCH_ENGINE_TYPE_NON_INDEXED,
-                                 recursive, child))
+        if (recursive_flag != NAUTILUS_QUERY_RECURSIVE_NEVER &&
+            g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)
+        {
+            if (recursive_flag == NAUTILUS_QUERY_RECURSIVE_ALWAYS)
+            {
+                recursive = TRUE;
+            }
+            else if (recursive_flag == NAUTILUS_QUERY_RECURSIVE_LOCAL_ONLY)
+            {
+                g_autoptr (GFileInfo) file_system_info = NULL;
+
+                file_system_info = g_file_query_filesystem_info (child,
+                                                                 G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE,
+                                                                 NULL, NULL);
+                if (file_system_info != NULL)
+                {
+                    recursive = !g_file_info_get_attribute_boolean (file_system_info,
+                                                                    G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE);
+                }
+            }
+        }
+
+        if (recursive)
         {
             id = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_ID_FILE);
             visited = FALSE;
@@ -504,12 +520,22 @@ search_thread_func (gpointer user_data)
     return NULL;
 }
 
+static void
+create_thread_timeout (gpointer user_data)
+{
+    NautilusSearchEngineSimple *simple = user_data;
+    g_autoptr (GThread) thread = NULL;
+
+    simple->create_thread_timeout_id = 0;
+    thread = g_thread_new ("nautilus-search-simple", search_thread_func, simple->active_search);
+}
+
 static void
 nautilus_search_engine_simple_start (NautilusSearchProvider *provider)
 {
     NautilusSearchEngineSimple *simple;
     SearchThreadData *data;
-    GThread *thread;
+    g_autoptr (GFile) location = NULL;
 
     simple = NAUTILUS_SEARCH_ENGINE_SIMPLE (provider);
 
@@ -522,12 +548,22 @@ nautilus_search_engine_simple_start (NautilusSearchProvider *provider)
 
     data = search_thread_data_new (simple, simple->query);
 
-    thread = g_thread_new ("nautilus-search-simple", search_thread_func, data);
     simple->active_search = data;
-
     g_object_notify (G_OBJECT (provider), "running");
 
-    g_thread_unref (thread);
+    location = nautilus_query_get_location (simple->query);
+    if (location == NULL)
+    {
+        /* Global search. Nothing for this engine to do.*/
+        finish_search_thread (data);
+        return;
+    }
+
+    g_queue_push_tail (data->directories, g_steal_pointer (&location));
+
+    simple->create_thread_timeout_id = g_timeout_add_once (CREATE_THREAD_DELAY_MS,
+                                                           create_thread_timeout,
+                                                           simple);
 }
 
 static void
@@ -539,6 +575,14 @@ nautilus_search_engine_simple_stop (NautilusSearchProvider *provider)
     {
         DEBUG ("Simple engine stop");
         g_cancellable_cancel (simple->active_search->cancellable);
+
+        if (simple->create_thread_timeout_id != 0)
+        {
+            /* Thread wasn't started, so we must call this directly from here.*/
+            finish_search_thread (simple->active_search);
+
+            g_clear_handle_id (&simple->create_thread_timeout_id, g_source_remove);
+        }
     }
 }
 
